<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

namespace WriteTextAI\WriteTextAI\Controller\Adminhtml\Categories\Edit;

use Magento\Backend\App\Action;
use Magento\Backend\App\Action\Context;
use Magento\Framework\HTTP\PhpEnvironment\RemoteAddress;
use WriteTextAI\WriteTextAI\Model\ApiManager;
use Magento\Framework\App\Config\ScopeConfigInterface;
use WriteTextAI\WriteTextAI\Helper\Store as StoreHelper;
use Magento\Framework\Filter\FilterManager;
use WriteTextAI\WriteTextAI\ViewModel\Premium;
use Magento\Framework\UrlInterface;
use Magento\Framework\Controller\ResultFactory;
use Magento\Store\Model\ScopeInterface;
use WriteTextAI\WriteTextAI\Model\OptionSource\Filter\Fields;
use WriteTextAI\WriteTextAI\Model\Categories\MarkReview;
use WriteTextAI\WriteTextAI\ViewModel\Settings;
use WriteTextAI\WriteTextAI\Helper\Html;
use WriteTextAI\WriteTextAI\Helper\Image;
use WriteTextAI\WriteTextAI\Model\AiCategoryManager;

class Generate extends Action
{
    public const ADMIN_RESOURCE = 'WriteTextAI_WriteTextAI::generate';

    /**
     * @var RemoteAddress
     */
    protected $remoteAddress;

    /**
     * @var ApiManager
     */
    protected $apiManager;

    /**
     * @var ScopeConfigInterface
     */
    protected $scopeConfig;

    /**
     * @var StoreHelper
     */
    protected $storeHelper;

    /**
     * @var FilterManager
     */
    protected $filterManager;

    /**
     * @var Premium
     */
    protected $premium;

    /**
     * @var UrlInterface
     */
    protected $urlBuilder;

    /**
     * @var MarkReview
     */
    protected $markReview;

    /**
     * @var Settings
     */
    protected $settings;

    /**
     * @var Html
     */
    protected $htmlHelper;

    /**
     * @var Image
     */
    protected $imageHelper;

    /**
     * @var AiCategoryManager
     */
    protected $aiCategoryManager;
    
    /**
     * Constructor
     *
     * @param Context $context
     * @param RemoteAddress $remoteAddress
     * @param ApiManager $apiManager
     * @param ScopeConfigInterface $scopeConfig
     * @param StoreHelper $storeHelper
     * @param FilterManager $filterManager
     * @param Premium $premium
     * @param UrlInterface $urlBuilder
     * @param MarkReview $markReview
     * @param Settings $settings
     * @param Html $htmlHelper
     * @param Image $imageHelper
     * @param AiCategoryManager $aiCategoryManager
     */
    public function __construct(
        Context $context,
        RemoteAddress $remoteAddress,
        ApiManager $apiManager,
        ScopeConfigInterface $scopeConfig,
        StoreHelper $storeHelper,
        FilterManager $filterManager,
        Premium $premium,
        UrlInterface $urlBuilder,
        MarkReview $markReview,
        Settings $settings,
        Html $htmlHelper,
        Image $imageHelper,
        AiCategoryManager $aiCategoryManager
    ) {
        $this->remoteAddress = $remoteAddress;
        $this->apiManager = $apiManager;
        $this->scopeConfig = $scopeConfig;
        $this->storeHelper = $storeHelper;
        $this->filterManager = $filterManager;
        $this->premium = $premium;
        $this->urlBuilder = $urlBuilder;
        $this->markReview = $markReview;
        $this->settings = $settings;
        $this->htmlHelper = $htmlHelper;
        $this->imageHelper = $imageHelper;
        $this->aiCategoryManager = $aiCategoryManager;

        parent::__construct($context);
    }

    /**
     * Execute action
     *
     * @return \Magento\Framework\Controller\Result\Json
     */
    public function execute()
    {
        $response = $this->resultFactory->create(ResultFactory::TYPE_JSON);
        
        try {
            $params = $this->getRequest()->getParams();
            
            $country =  $this->scopeConfig->getValue(
                'general/country/default',
                ScopeInterface::SCOPE_STORE,
                $params['store_id']
            );
            $language = $this->storeHelper->getFormattedLanguage($params['store_id']);
            $ipAddress = $this->remoteAddress->getRemoteAddress();

            $body = [
                "type" => "Category",
                "storeId" => $params['store_id'],
                "language" => $language,
                "countries" => [$country],
                "ipAddress" => $ipAddress,
                "texts" => [$this->getTextsParams($params)]
            ];

            $apiResponse = $this->apiManager->generate(json_encode($body));

            $this->aiCategoryManager->saveDate($params['category_id'], $params['store_id'], 'edited');
            $this->aiCategoryManager->saveDate($params['category_id'], $params['store_id'], 'generated');

            $result = [
                'success' => true,
                'is_premium' => $this->premium->isPremium(),
                'is_free_premium_credits' => $this->premium->getFreePremiumCredits(),
                'is_already_premium' => $this->premium->getFreeCreditsAlreadyPremium(),
                'reviewed' => $this->markReview->getReviewed($params['category_id'], $params['store_id']),
                'api_request' => $body,
                'api_response' => $apiResponse
            ];
        } catch (\WriteTextAI\WriteTextAI\Exception\UnauthorizedException $e) {
            $result = [
                'success' => false,
                'unauthorized' => true,
                'login_url' => $this->urlBuilder->getUrl('wtai/setup/index'),
                'message' => $e->getMessage()
            ];
        } catch (\Exception $e) {
            $result = [
                'success' => false,
                'is_premium' => $this->premium->isPremium(),
                'is_free_premium_credits' => $this->premium->getFreePremiumCredits(),
                'is_already_premium' => $this->premium->getFreeCreditsAlreadyPremium(),
                'message' => $e->getMessage(),
                'api_request' => $body ?? [],
                'api_response' => $apiResponse ?? []
            ];
        }

        $response->setData($result);

        return $response;
    }

    /**
     * Get texts params
     *
     * @param array $params
     * @return array
     */
    private function getTextsParams($params)
    {
        $categoryName = $params['name'];
        $texts = $params['target_keywords'] ?? [];
        $targetKeywords = array_filter($texts, function ($keyword) use ($categoryName) {
            return $keyword !== $categoryName;
        });
        $targetKeywords = array_values($targetKeywords);
        
        $textsParams = [
            "name" => $params['name'],
            "recordId" => $params['category_id'],
            "options" => 1,
            "autoselectFirst" => true,
            "keywords" => $targetKeywords,
            "semanticKeywords" => $params['semantic_keywords'] ?? [],
            "keywordAnalysisViews" => $params['keyword_analysis_views']
        ];

        if (filter_var($params['other_details_checked'], FILTER_VALIDATE_BOOLEAN)) {
            $textsParams['otherDetails'] = $this->filterManager->stripTags($params['other_details']);
        }

        if (in_array('custom', $params['tones'])) {
            $textsParams['customTone'] = $params['custom_tone'];
        } else {
            $textsParams['tones'] = $params['tones'];
        }

        if ($params['style'] === 'custom') {
            $textsParams['customStyle'] = $params['custom_style'];
        } else {
            $textsParams['style'] = $params['style'];
        }

        if (!empty($params['audience'])) {
            $textsParams['audiences'] = $params['audience'];
        }

        if (isset($params['custom_audience'])) {
            $textsParams['customAudience'] = $params['custom_audience'];
        }

        $rules = $this->settings->getSettings('rules');
        $credits = $this->settings->getSettings('credits');

        $maxInputChar = ((($rules['maxOutputWords'] / $credits['wordsPerCredit']) - 1
            ) * $rules['additionalReferenceTextLength']
        ) + $rules['maxReferenceTextLength'];

        $fields = [];
        foreach ($params['selected_fields'] as $selectedField) {
            $field = [];
            $field['field'] = Fields::CATEGORY_MAPPING[$selectedField];
            $field['currentValue'] = $params['current_values'][$selectedField];
            
            if (Fields::CATEGORY_MAPPING[$selectedField] === Fields::CATEGORY_DESCRIPTION) {
                $field["minWords"] = $params['min_words'];
                $field["maxWords"] = $params['max_words'];
            }

            if (filter_var($params['rewrite'], FILTER_VALIDATE_BOOLEAN)) {
                if ($field['currentValue']) {
                    $rewriteText = substr($field['currentValue'], 0, $maxInputChar);
                    $field['rewriteText'] = $this->htmlHelper->removeDisallowedHtml($rewriteText);
                    $fields[] = $field;
                }
            } else {
                $fields[] = $field;
            }
        }
        $textsParams['fields'] = $fields;

        if (isset($params['products']) && is_array($params['products'])) {
            $products = [];
            foreach ($params['products'] as $product) {
                $productData = [
                    "id" => $product['value'] ?? '',
                    "name" => $product['label'] ?? '',
                    "description" => $product['description'] ?? ''
                ];

                if ($product['image_id']) {
                    $productData['image'] = $product['image_id'];
                }

                $products[] = $productData;
            }
            $textsParams['products'] = $products;
        }

        if ($params['category_image'] && filter_var($params['category_image_selected'], FILTER_VALIDATE_BOOLEAN)) {
            $textsParams['images'] = [
                $this->imageHelper->getCategoryImageId($params['category_id'], $params['store_id'])
            ];
        }

        return $textsParams;
    }
}
