<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

namespace WriteTextAI\WriteTextAI\Controller\Adminhtml\Categories\Image;

use Magento\Backend\App\Action;
use Magento\Backend\App\Action\Context;
use Magento\Framework\Controller\ResultFactory;
use WriteTextAI\WriteTextAI\ViewModel\Premium;
use Magento\Framework\HTTP\PhpEnvironment\RemoteAddress;
use WriteTextAI\WriteTextAI\Helper\Store as StoreHelper;
use WriteTextAI\WriteTextAI\Helper\Image;
use Magento\Framework\UrlInterface;

class Upload extends Action
{
    /**
     * @var Image
     */
    protected $imageHelper;

    /**
     * @var UrlInterface
     */
    protected $urlBuilder;

    /**
     * @var Premium
     */
    protected $premium;

    /**
     * @var RemoteAddress
     */
    protected $remoteAddress;

    /**
     * @var StoreHelper
     */
    protected $storeHelper;
    
    /**
     * Constructor
     *
     * @param Context $context
     * @param Image $imageHelper
     * @param UrlInterface $urlBuilder
     * @param Premium $premium
     * @param RemoteAddress $remoteAddress
     * @param StoreHelper $storeHelper
     */
    public function __construct(
        Context $context,
        Image $imageHelper,
        UrlInterface $urlBuilder,
        Premium $premium,
        RemoteAddress $remoteAddress,
        StoreHelper $storeHelper
    ) {
        $this->imageHelper = $imageHelper;
        $this->urlBuilder = $urlBuilder;
        $this->premium = $premium;
        $this->remoteAddress = $remoteAddress;
        $this->storeHelper = $storeHelper;
        parent::__construct($context);
    }

    /**
     * Execute action
     *
     * @return \Magento\Framework\Controller\Result\Json
     */
    public function execute()
    {
        $response = $this->resultFactory->create(ResultFactory::TYPE_JSON);

        try {
            $params = $this->getRequest()->getPost();

            $categoryImageUpload = true;
            if (isset($params['category_image'])) {
                $categoryImageUpload = $this->uploadCategoryImage($params);
            }

            $productImagesUpload = $this->uploadProductImages($params);

            $result = [
                'is_premium' => $this->premium->isPremium(),
                'success' => empty($failedImages),
                'category_image' => $categoryImageUpload,
                'products' => $productImagesUpload
            ];
        } catch (\WriteTextAI\WriteTextAI\Exception\UnauthorizedException $e) {
            $result = [
                'success' => false,
                'unauthorized' => true,
                'login_url' => $this->urlBuilder->getUrl('wtai/setup/index'),
                'message' => $e->getMessage()
            ];
        } catch (\Exception $e) {
            $result = [
                'success' => false,
                'is_premium' => $this->premium->isPremium(),
                'message' => $e->getMessage()
            ];
        }

        $response->setData($result);

        return $response;
    }

    /**
     * Upload category image
     *
     * @param array $params
     * @return bool
     */
    private function uploadCategoryImage($params)
    {
        try {
            $this->imageHelper->uploadImage(
                $this->remoteAddress->getRemoteAddress(),
                $params['category_image'],
                [
                    "storeId" => $params['store_id'],
                    "language" => $this->storeHelper->getFormattedLanguage($params['store_id']),
                    "imageId" => $this->imageHelper->getCategoryImageId($params['category_id'], $params['store_id'])
                ],
                true
            );
            return true;
        } catch (\WriteTextAI\WriteTextAI\Exception\ImageUnsupported $e) {
            return false;
        } catch (\WriteTextAI\WriteTextAI\Exception\ImageInvalid $e) {
            return false;
        }
    }

    /**
     * Upload product images
     *
     * @param array $params
     * @return array
     */
    private function uploadProductImages($params)
    {
        $success = [];
        $failed = [];
        if (isset($params['products']) && !empty($params['products'])) {
            foreach ($params['products'] as $product) {
                if (!$product['image_id']) {
                    continue;
                }

                try {
                    $this->imageHelper->getImageApiId(
                        $params['store_id'],
                        $this->remoteAddress->getRemoteAddress(),
                        $this->storeHelper->getFormattedLanguage($params['store_id']),
                        $product['image_id'],
                        $product['image']
                    );
                    $success[] = $product;
                } catch (\WriteTextAI\WriteTextAI\Exception\ImageUnsupported $e) {
                    $failed[] = $product;
                } catch (\WriteTextAI\WriteTextAI\Exception\ImageInvalid $e) {
                    $failed[] = $product;
                }
            }
        }

        return [
            'failed' => $failed,
            'success' => $success
        ];
    }
}
