<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

namespace WriteTextAI\WriteTextAI\Model;

use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Store\Model\ScopeInterface;
use WriteTextAI\WriteTextAI\Model\ProductKeywordIdeasFiltersFactory;
use WriteTextAI\WriteTextAI\Api\ProductKeywordIdeasFiltersRepositoryInterface;
use WriteTextAI\WriteTextAI\ViewModel\Settings;

class KeywordIdeaFiltersManager
{
    /**
     * @var ProductKeywordIdeasFiltersFactory
     */
    protected $productKeywordIdeasFiltersFactory;

    /**
     * @var ProductKeywordIdeasFiltersRepositoryInterface
     */
    protected $productKeywordIdeasFiltersRepository;

    /**
     * @var ScopeConfigInterface
     */
    protected $scopeConfig;

    /**
     * @var Settings
     */
    protected $settings;

    /**
     * @param ProductKeywordIdeasFiltersFactory $productKeywordIdeasFiltersFactory
     * @param ProductKeywordIdeasFiltersRepositoryInterface $productKeywordIdeasFiltersRepository
     * @param ScopeConfigInterface $scopeConfig
     * @param Settings $settings
     */
    public function __construct(
        ProductKeywordIdeasFiltersFactory $productKeywordIdeasFiltersFactory,
        ProductKeywordIdeasFiltersRepositoryInterface $productKeywordIdeasFiltersRepository,
        ScopeConfigInterface $scopeConfig,
        Settings $settings
    ) {
        $this->productKeywordIdeasFiltersFactory = $productKeywordIdeasFiltersFactory;
        $this->productKeywordIdeasFiltersRepository = $productKeywordIdeasFiltersRepository;
        $this->scopeConfig = $scopeConfig;
        $this->settings = $settings;
    }

    /**
     * Get available countries
     *
     * @return array
     */
    public function getAvailableCountries()
    {
        $locations = $this->settings->getSettings('locations');
        if (!is_array($locations)) {
            $locations = json_decode($locations, true);
        }

        return $locations;
    }

    /**
     * Get last used filters
     *
     * @param int $productId
     * @param int $storeId
     * @param string $entityType
     *
     * @return array
     */
    public function getKeywordIdeasFilters($productId, $storeId, $entityType = 'product')
    {
        $filters = $this->productKeywordIdeasFiltersFactory->create();

        $filters = $filters->getCollection()
            ->addFieldToFilter('product_id', $productId)
            ->addFieldToFilter('store_id', $storeId)
            ->addFieldToFilter('entity_type', $entityType)
            ->getFirstItem();

        if ($filters->getId()) {
            $competition = array_map(function ($value) {
                $mapping = [
                    1 => (string) __("LOW"),
                    2 => (string) __("MEDIUM"),
                    3 => (string) __("HIGH")
                ];

                return $mapping[$value];
            }, explode(',', (string) $filters->getCompetition()));
            
            return [
                'competition' => $competition,
                'search_volume_minimum' => (int) $filters->getSearchVolumeMinimum(),
                'search_volume_maximum' => $filters->getSearchVolumeMaximum() === null
                    ? null : (int) $filters->getSearchVolumeMaximum(),
                'competition_sort' => $filters->getCompetitionSort()
                    ? filter_var($filters->getCompetitionSort(), FILTER_VALIDATE_BOOLEAN) : '',
                'volume_sort' => $filters->getVolumeSort()
                    ? filter_var($filters->getVolumeSort(), FILTER_VALIDATE_BOOLEAN) : '',
                'keyword_sort' => $filters->getKeywordSort()
                    ? filter_var($filters->getKeywordSort(), FILTER_VALIDATE_BOOLEAN) : ''
            ];
        } else {
            return [
                'competition' => [
                    __("LOW"),
                    __("MEDIUM"),
                    __("HIGH")
                ],
                'search_volume_minimum' => 0,
                'search_volume_maximum' => null,
                'competition_sort' => '',
                'volume_sort' => '',
                'keyword_sort' => true
            ];
        }
    }

    /**
     * Get default location
     *
     * @param int $storeId
     * @return string
     */
    public function getLocationCode($storeId)
    {
        $countryCode = $this->getCountryCode($storeId);
        $locations = $this->getAvailableCountries();

        foreach ($locations as $location) {
            if ($location['country_iso_code'] == $countryCode) {
                return $location['location_code'];
            }
        }

        return 2840; // Default to United States
    }

    /**
     * Get country code
     *
     * @param int $storeId
     * @return string
     */
    public function getCountryCode($storeId)
    {
        $countryCode =  $this->scopeConfig->getValue(
            'general/country/default',
            ScopeInterface::SCOPE_STORE,
            $storeId
        );

        return $countryCode;
    }

    /**
     * Save last used filters
     *
     * @param int $storeId
     * @param int $productId
     * @param array $filterByCompetition
     * @param int $filterBySearchVolumeMinimum
     * @param int $filterBySearchVolumeMaximum
     * @param string $competitionSort
     * @param string $volumeSort
     * @param string $keywordSort
     * @param string $entityType
     *
     * @return void
     */
    public function saveProductKeywordIdeasFilters(
        $storeId,
        $productId,
        $filterByCompetition,
        $filterBySearchVolumeMinimum,
        $filterBySearchVolumeMaximum,
        $competitionSort,
        $volumeSort,
        $keywordSort,
        $entityType = 'product'
    ) {
        $filters = $this->productKeywordIdeasFiltersFactory->create();

        $filterByCompetition = array_map(function ($value) {
            $mapping = [
                (string) __("LOW") => 1,
                (string) __("MEDIUM") => 2,
                (string) __("HIGH") => 3
            ];

            return $mapping[$value];
        }, $filterByCompetition);
        $filterByCompetition = implode(',', (array) $filterByCompetition);

        $existing = $filters->getCollection()
            ->addFieldToFilter('product_id', $productId)
            ->addFieldToFilter('store_id', $storeId)
            ->addFieldToFilter('entity_type', $entityType)
            ->getFirstItem();

        if ($existing->getId()) {
            // Update the existing record
            $existing->setCompetition($filterByCompetition);
            $existing->setSearchVolumeMinimum($filterBySearchVolumeMinimum);
            $existing->setCompetitionSort($competitionSort);
            $existing->setVolumeSort($volumeSort);
            $existing->setKeywordSort($keywordSort);
            $existing->setSearchVolumeMaximum(
                $filterBySearchVolumeMaximum === '' ? null : $filterBySearchVolumeMaximum
            );

            $filters = $existing;
        } else {
            // Record not found, create a new one
            $data = [
                "product_id" => $productId,
                "store_id" => $storeId,
                "competition" => $filterByCompetition,
                "search_volume_minimum" => $filterBySearchVolumeMinimum,
                "competition_sort" => $competitionSort,
                "volume_sort" => $volumeSort,
                "keyword_sort" => $keywordSort,
                "entity_type" => $entityType
            ];
            
            if ($filterBySearchVolumeMaximum) {
                $data['search_volume_maximum'] = $filterBySearchVolumeMaximum;
            }

            $filters->setData($data);
        }

        $this->productKeywordIdeasFiltersRepository->save($filters);
    }

    /**
     * Delete filter
     *
     * @param int $storeId
     * @param int $productId
     * @param string $entityType
     *
     * @return void
     */
    public function deleteFilters($storeId, $productId, $entityType = 'product')
    {
        $filters = $this->productKeywordIdeasFiltersFactory->create();

        $existing = $filters->getCollection()
            ->addFieldToFilter('product_id', $productId)
            ->addFieldToFilter('store_id', $storeId)
            ->addFieldToFilter('entity_type', 'product')
            ->getFirstItem();

        if ($existing->getId()) {
            $this->productKeywordIdeasFiltersRepository->delete($existing);
        }
    }
}
