<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

namespace WriteTextAI\WriteTextAI\Model;

use WriteTextAI\WriteTextAI\Model\ProductKeywordRankedFiltersFactory;
use WriteTextAI\WriteTextAI\Api\ProductKeywordRankedFiltersRepositoryInterface;

class KeywordRankedFiltersManager
{
    /**
     * @var ProductKeywordRankedFiltersFactory
     */
    protected $productKeywordRankedFiltersFactory;

    /**
     * @var ProductKeywordRankedFiltersRepositoryInterface
     */
    protected $productKeywordRankedFiltersRepository;

    /**
     * @param ProductKeywordRankedFiltersFactory $productKeywordRankedFiltersFactory
     * @param ProductKeywordRankedFiltersRepositoryInterface $productKeywordRankedFiltersRepository
     */
    public function __construct(
        ProductKeywordRankedFiltersFactory $productKeywordRankedFiltersFactory,
        ProductKeywordRankedFiltersRepositoryInterface $productKeywordRankedFiltersRepository
    ) {
        $this->productKeywordRankedFiltersFactory = $productKeywordRankedFiltersFactory;
        $this->productKeywordRankedFiltersRepository = $productKeywordRankedFiltersRepository;
    }

    /**
     * Get last used filters
     *
     * @param int $productId
     * @param int $storeId
     * @param string $entityType
     *
     * @return array
     */
    public function getKeywordRankedFilters($productId, $storeId, $entityType = 'product')
    {
        $filters = $this->productKeywordRankedFiltersFactory->create();

        $filters = $filters->getCollection()
            ->addFieldToFilter('product_id', $productId)
            ->addFieldToFilter('store_id', $storeId)
            ->addFieldToFilter('entity_type', $entityType)
            ->getFirstItem();

        if ($filters->getId()) {
            $competition = array_map(function ($value) {
                $mapping = [
                    1 => (string) __("LOW"),
                    2 => (string) __("MEDIUM"),
                    3 => (string) __("HIGH")
                ];

                return $mapping[$value];
            }, explode(',', (string) $filters->getCompetition()));
            
            return [
                'competition' => $competition,
                'search_volume_minimum' => (int) $filters->getSearchVolumeMinimum(),
                'search_volume_maximum' => $filters->getSearchVolumeMaximum() === null
                    ? null : (int) $filters->getSearchVolumeMaximum(),
                'competition_sort' => $filters->getCompetitionSort()
                    ? filter_var($filters->getCompetitionSort(), FILTER_VALIDATE_BOOLEAN) : '',
                'volume_sort' => $filters->getVolumeSort()
                    ? filter_var($filters->getVolumeSort(), FILTER_VALIDATE_BOOLEAN) : '',
                'rank_sort' => $filters->getRankSort()
                    ? filter_var($filters->getRankSort(), FILTER_VALIDATE_BOOLEAN) : '',
                'intent_sort' => $filters->getIntentSort()
                    ? filter_var($filters->getIntentSort(), FILTER_VALIDATE_BOOLEAN) : ''
            ];
        } else {
            return [
                'competition' => [
                    __("LOW"),
                    __("MEDIUM"),
                    __("HIGH")
                ],
                'search_volume_minimum' => 0,
                'search_volume_maximum' => null,
                'competition_sort' => '',
                'volume_sort' => '',
                'rank_sort' => '',
                'intent_sort' => ''
            ];
        }
    }

    /**
     * Save last used filters
     *
     * @param int $storeId
     * @param int $productId
     * @param array $filterByCompetition
     * @param int $filterBySearchVolumeMinimum
     * @param int $filterBySearchVolumeMaximum
     * @param string $competitionSort
     * @param string $volumeSort
     * @param string $rankSort
     * @param string $intentSort
     * @param string $entityType
     *
     * @return void
     */
    public function saveKeywordRankedFilters(
        $storeId,
        $productId,
        $filterByCompetition,
        $filterBySearchVolumeMinimum,
        $filterBySearchVolumeMaximum,
        $competitionSort,
        $volumeSort,
        $rankSort,
        $intentSort,
        $entityType = 'product'
    ) {
        $filters = $this->productKeywordRankedFiltersFactory->create();

        $filterByCompetition = array_map(function ($value) {
            $mapping = [
                (string) __("LOW") => 1,
                (string) __("MEDIUM") => 2,
                (string) __("HIGH") => 3
            ];

            return $mapping[$value];
        }, $filterByCompetition);
        $filterByCompetition = implode(',', (array) $filterByCompetition);

        $existing = $filters->getCollection()
            ->addFieldToFilter('product_id', $productId)
            ->addFieldToFilter('store_id', $storeId)
            ->addFieldToFilter('entity_type', $entityType)
            ->getFirstItem();

        if ($existing->getId()) {
            // Update the existing record
            $existing->setCompetition($filterByCompetition);
            $existing->setSearchVolumeMinimum($filterBySearchVolumeMinimum);
            $existing->setCompetitionSort($competitionSort);
            $existing->setVolumeSort($volumeSort);
            $existing->setRankSort($rankSort);
            $existing->setIntentSort($intentSort);
            $existing->setSearchVolumeMaximum(
                $filterBySearchVolumeMaximum === '' ? null : $filterBySearchVolumeMaximum
            );

            $filters = $existing;
        } else {
            // Record not found, create a new one
            $data = [
                "product_id" => $productId,
                "store_id" => $storeId,
                "competition" => $filterByCompetition,
                "search_volume_minimum" => $filterBySearchVolumeMinimum,
                "competition_sort" => $competitionSort,
                "volume_sort" => $volumeSort,
                "rank_sort" => $rankSort,
                "intent_sort" => $intentSort,
                "entity_type" => $entityType
            ];
            
            if ($filterBySearchVolumeMaximum) {
                $data['search_volume_maximum'] = $filterBySearchVolumeMaximum;
            }

            $filters->setData($data);
        }

        $this->productKeywordRankedFiltersRepository->save($filters);
    }

    /**
     * Delete filter
     *
     * @param int $storeId
     * @param int $productId
     * @param string $entityType
     *
     * @return void
     */
    public function deleteFilters($storeId, $productId, $entityType = 'product')
    {
        $filters = $this->productKeywordRankedFiltersFactory->create();

        $existing = $filters->getCollection()
            ->addFieldToFilter('product_id', $productId)
            ->addFieldToFilter('store_id', $storeId)
            ->addFieldToFilter('entity_type', $entityType)
            ->getFirstItem();

        if ($existing->getId()) {
            $this->productKeywordRankedFiltersRepository->delete($existing);
        }
    }
}
