<?php
/**
 * @author    1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package   WriteTextAI_WriteTextAI
 */

namespace WriteTextAI\WriteTextAI\Model\Magento;

use Magento\Catalog\Model\ResourceModel\Product\Collection;
use Magento\Catalog\Model\ResourceModel\Product\CollectionFactory;
use Magento\Catalog\Model\ResourceModel\Category\CollectionFactory as CategoryCollectionFactory;
use Magento\Catalog\Model\Category as MagentoCategory;
use Magento\Catalog\Model\Product\Visibility;
use Magento\Catalog\Model\Product\Attribute\Source\Status;
use Magento\Catalog\Helper\Image as ImageHelper;
use WriteTextAI\WriteTextAI\Helper\Image as WtaImageHelper;
use WriteTextAI\WriteTextAI\Helper\Data;

class Category
{
    /**
     * @var MagentoCategory
     */
    protected $category;
    
    /**
     * @var CategoryCollectionFactory
     */
    protected $categoryCollectionFactory;

    /**
     * @var CollectionFactory
     */
    protected $productCollectionFactory;

    /**
     * @var ImageHelper
     */
    protected $imageHelper;

    /**
     * @var WtaImageHelper
     */
    protected $wtaImageHelper;

    /**
     * @var Data
     */
    protected $helper;

    /**
     * Constructor
     *
     * @param CategoryCollectionFactory $categoryCollectionFactory
     * @param CollectionFactory $productCollectionFactory
     * @param ImageHelper $imageHelper
     * @param WtaImageHelper $wtaImageHelper
     * @param Data $helper
     */
    public function __construct(
        CategoryCollectionFactory $categoryCollectionFactory,
        CollectionFactory $productCollectionFactory,
        ImageHelper $imageHelper,
        WtaImageHelper $wtaImageHelper,
        Data $helper
    ) {
        $this->categoryCollectionFactory = $categoryCollectionFactory;
        $this->productCollectionFactory = $productCollectionFactory;
        $this->imageHelper = $imageHelper;
        $this->wtaImageHelper = $wtaImageHelper;
        $this->helper = $helper;
    }
    
    /**
     * Get category object
     *
     * @param int $categoryId
     * @param int $storeId
     *
     * @return MagentoCategory
     */
    public function getCategory($categoryId, $storeId)
    {
        if ($this->category
            && $this->category->getId() === $categoryId
            && $this->category->getStoreId() === $storeId
        ) {
            return $this->category;
        }

        $collection = $this->categoryCollectionFactory->create();
        $collection->setStoreId($storeId);
        $collection->addAttributeToSelect('*');
        $collection->addFieldToFilter('entity_id', $categoryId);
        
        $this->category = $collection->getFirstItem();

        return $this->category;
    }

    /**
     * Get all children categories IDs
     *
     * @param boolean $asArray
     * @param int $categoryId
     * @param int $storeId
     *
     * @return array|string
     */
    public function getAllChildren($asArray = false, $categoryId = null, $storeId = null)
    {
        $category;
        if ($this->category
            && $this->category->getId() === $categoryId
            && $this->category->getStoreId() === $storeId
        ) {
            $category = $this->category;
        } else {
            $category = $this->getCategory($categoryId, $storeId);
        }
        
        if ($category->getIsAnchor()) {
            return $category->getAllChildren($asArray);
        } else {
            return $asArray ? [$categoryId] : $categoryId;
        }
    }

    /**
     * Get all children of root category
     *
     * @param int $categoryId
     * @param int $storeId
     *
     * @return array|string
     */
    public function getRootChildren($categoryId, $storeId)
    {
        $category;
        if ($this->category
            && $this->category->getId() === $categoryId
            && $this->category->getStoreId() === $storeId
        ) {
            $category = $this->category;
        } else {
            $category = $this->getCategory($categoryId, $storeId);
        }
        
        $collection = $this->categoryCollectionFactory->create();
        $collection->addAttributeToFilter('path', ['like' => $category->getPath() . '/%']);
        return $collection->getAllIds();
    }

    /**
     * Get product collection
     *
     * @param array $categoryIds
     * @param int $storeId
     * @param int $page
     * @param string $search
     * @param int $limit
     *
     * @return array
     */
    public function getProductCollection($categoryIds, $storeId, $page = 1, $search = '', $limit = 10)
    {
        $collection = $this->productCollectionFactory->create();
        $collection->setStoreId($storeId);
        $collection->addAttributeToSelect('*');
        $collection->addAttributeToSort('name', 'ASC');
        $collection->addCategoriesFilter(['in' => $categoryIds]);

        if ($search) {
            $collection->addFieldToFilter('name', ['like' => '%' . $search . '%']);
        }

        $productIds = $collection->getAllIds();

        $collection->setPageSize($limit * $page);
        $collection->addMediaGalleryData();

        $options = [];
        $mappingSettings = $this->helper->getMappingSettings();
        foreach ($collection as $product) {
            $name = $product->getName();
            $image = $this->imageHelper->init($product, 'product_listing_thumbnail')
                ->setImageFile($product->getThumbnail())
                ->getUrl();
            $imageId = $this->wtaImageHelper->getImageValue($product);
            $description = $product->getData($mappingSettings['product_description']);
            $status = $product->getStatus() == Status::STATUS_ENABLED ? __('enabled') : __('disabled');
            $options[] = [
                'value' => [
                    'value' => $product->getId(),
                    'label' => $name,
                    'image' => $image,
                    'image_id' => $imageId,
                    'description' => $description,
                    'status' => sprintf("(%s)", $status)
                ],
                'label' => $name,
                'image' => $image,
                'image_id' => $imageId,
                'description' => $description,
                'status' => sprintf("(%s)", $status)
            ];
        }

        return [
            'all_options' => $productIds,
            'options' => $options,
            'total' => $collection->getSize()
        ];
    }
}
