<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

namespace WriteTextAI\WriteTextAI\Model;

use WriteTextAI\WriteTextAI\Model\UserFieldsSelectedFactory;
use WriteTextAI\WriteTextAI\Api\UserFieldsSelectedRepositoryInterface;
use WriteTextAI\WriteTextAI\Model\Api\Session as ApiSession;
use WriteTextAI\WriteTextAI\Helper\Data;

class UserFieldsSelectedManager
{
    public const ACTION_TYPE_HIGHLIGHTED = 'highlighted';

    public const ACTION_TYPE_HIGHLIGHT_PRONOUNS = 'highlight_pronouns';

    public const ACTION_TYPE_ENABLE_COMPARISON_POPUP = 'enable_comparison_popup';

    public const ACTION_TYPE_CUSTOM_TONE = 'custom_tone';

    public const ACTION_TYPE_CUSTOM_STYLE = 'custom_style';

    public const ACTION_TYPE_STYLE = 'style';

    public const ACTION_TYPE_TONES = 'tones';

    public const ACTION_TYPE_AUDIENCES = 'audiences';

    public const ACTION_TYPE_SINGLE_EDITOR_FIELDS = 'single_editor';

    public const ACTION_TYPE_BULK_GENERATE_FIELDS = 'generate';

    public const ACTION_TYPE_BULK_TRANSFER_FIELDS = 'transfer';

    public const ACTION_TYPE_GUIDE_STEPS = 'guide_steps';

    public const ACTION_TYPE_GALLERY = 'gallery';

    public const ACTION_TYPE_POPUP_BLOCKER_NOTICE = 'popup_blocker_notice';

    public const ACTION_TYPE_CATEGORY_IMAGE = 'category_image';

    public const ACTION_TYPE_USE_KEYWORDS = 'use_keywords';

    public const ACTION_TYPE_THUMBNAIL = 'thumbnail';

    /**
     * @var UserFieldsSelectedFactory
     */
    protected $userFieldsSelectedFactory;

    /**
     * @var UserFieldsSelectedRepositoryInterface
     */
    protected $userFieldsSelectedRepository;

    /**
     * @var ApiSession
     */
    protected $apiSession;

    /**
     * @var Data
     */
    protected $helper;

    /**
     * Constructor
     *
     * @param UserFieldsSelectedFactory $userFieldsSelectedFactory
     * @param UserFieldsSelectedRepositoryInterface $userFieldsSelectedRepository
     * @param ApiSession $apiSession
     * @param Data $helper
     */
    public function __construct(
        UserFieldsSelectedFactory $userFieldsSelectedFactory,
        UserFieldsSelectedRepositoryInterface $userFieldsSelectedRepository,
        ApiSession $apiSession,
        Data $helper
    ) {
        $this->userFieldsSelectedFactory = $userFieldsSelectedFactory;
        $this->userFieldsSelectedRepository = $userFieldsSelectedRepository;
        $this->apiSession = $apiSession;
        $this->helper = $helper;
    }

    /**
     * Get user's last text types selected
     *
     * @param string $actionType
     * @param string $entityType
     *
     * @return array
     */
    public function getUserFieldsSelected($actionType, $entityType = 'product')
    {
        $userId = $this->getUser()->getId();

        $userFieldsSelected = $this->userFieldsSelectedFactory->create();

        $userFieldsSelected = $userFieldsSelected->getCollection()
            ->addFieldToFilter('user_id', $userId)
            ->addFieldToFilter('action_type', $actionType)
            ->addFieldToFilter('entity_type', $entityType)
            ->getFirstItem();

        $customOpenGraph = $this->helper->getCustomOpenGraph();
        
        if ($userFieldsSelected->getId()) {
            $fields = (string) $userFieldsSelected->getFields();

            switch ($actionType) {
                case self::ACTION_TYPE_HIGHLIGHTED:
                case self::ACTION_TYPE_GALLERY:
                case self::ACTION_TYPE_HIGHLIGHT_PRONOUNS:
                case self::ACTION_TYPE_ENABLE_COMPARISON_POPUP:
                case self::ACTION_TYPE_CUSTOM_TONE:
                case self::ACTION_TYPE_CUSTOM_STYLE:
                case self::ACTION_TYPE_STYLE:
                case self::ACTION_TYPE_GUIDE_STEPS:
                case self::ACTION_TYPE_POPUP_BLOCKER_NOTICE:
                case self::ACTION_TYPE_CATEGORY_IMAGE:
                case self::ACTION_TYPE_USE_KEYWORDS:
                case self::ACTION_TYPE_THUMBNAIL:
                    return $fields;
                case self::ACTION_TYPE_SINGLE_EDITOR_FIELDS:
                case self::ACTION_TYPE_BULK_GENERATE_FIELDS:
                case self::ACTION_TYPE_BULK_TRANSFER_FIELDS:
                case self::ACTION_TYPE_TONES:
                case self::ACTION_TYPE_AUDIENCES:
                    if (!$fields) {
                        return [];
                    }
                    
                    $arrayFields = explode(",", (string) $fields);

                    /**
                     * If custom open graph is disabled, remove open graph from the selected fields
                     */
                    if (!$customOpenGraph) {
                        foreach (array_keys($arrayFields, 'open_graph', true) as $key) {
                            unset($arrayFields[$key]);
                        }
                    }
                    
                    return $arrayFields;
            }
        } else {
            switch ($actionType) {
                case self::ACTION_TYPE_HIGHLIGHTED:
                case self::ACTION_TYPE_GALLERY:
                case self::ACTION_TYPE_HIGHLIGHT_PRONOUNS:
                case self::ACTION_TYPE_CATEGORY_IMAGE:
                case self::ACTION_TYPE_USE_KEYWORDS:
                case self::ACTION_TYPE_THUMBNAIL:
                    $defaultProductAttributes = $this->helper->getAttributesSettings('attributes');
                    if ($defaultProductAttributes) {
                        $selectedAttributes = explode(",", (string) $defaultProductAttributes);
                        if (in_array('thumbnail', $selectedAttributes)) {
                            return "true";
                        }
                    }
                    return "false";
                case self::ACTION_TYPE_ENABLE_COMPARISON_POPUP:
                    return "false";
                case self::ACTION_TYPE_CUSTOM_TONE:
                case self::ACTION_TYPE_CUSTOM_STYLE:
                case self::ACTION_TYPE_STYLE:
                    return "";
                case self::ACTION_TYPE_TONES:
                case self::ACTION_TYPE_AUDIENCES:
                    return [];
                case self::ACTION_TYPE_GUIDE_STEPS:
                    return false;
                case self::ACTION_TYPE_POPUP_BLOCKER_NOTICE:
                    return true;
                case self::ACTION_TYPE_BULK_GENERATE_FIELDS:
                    $fields = [
                        'page_title',
                        'page_description',
                        'product_description',
                        'short_product_description',
                        'alt_text'
                    ];

                    if ($customOpenGraph) {
                        $fields[] = 'open_graph';
                    }

                    return $fields;
                case self::ACTION_TYPE_SINGLE_EDITOR_FIELDS:
                case self::ACTION_TYPE_BULK_TRANSFER_FIELDS:
                    if ($entityType == 'category') {
                        $fields = [
                            'page_title',
                            'page_description',
                            'category_description'
                        ];
                    } else {
                        $fields = [
                            'page_title',
                            'page_description',
                            'product_description',
                            'short_product_description'
                        ];

                        if ($customOpenGraph) {
                            $fields[] = 'open_graph';
                        }
                    }

                    return $fields;
            }
        }
    }

    /**
     * Save user fields selected
     *
     * @param array $fields
     * @param string $actionType
     * @param string $entityType
     *
     * @return void
     */
    public function saveUserFieldsSelected($fields, $actionType, $entityType = 'product')
    {
        $userId = $this->getUser()->getId();

        $userFieldsSelected = $this->userFieldsSelectedFactory->create();

        $existing = $userFieldsSelected->getCollection()
            ->addFieldToFilter('user_id', $userId)
            ->addFieldToFilter('action_type', $actionType)
            ->addFieldToFilter('entity_type', $entityType)
            ->getFirstItem();

        if (is_array($fields) &&
            in_array($actionType, [
                self::ACTION_TYPE_TONES,
                self::ACTION_TYPE_AUDIENCES,
                self::ACTION_TYPE_SINGLE_EDITOR_FIELDS,
                self::ACTION_TYPE_BULK_GENERATE_FIELDS,
                self::ACTION_TYPE_BULK_TRANSFER_FIELDS
            ])
        ) {
            $fields = implode(",", $fields);
        }

        if ($existing->getId()) {
            // Update the existing record
            $existing->setFields($fields);
            $userFieldsSelected = $existing;
        } else {
            // Record not found, create a new one
            $userFieldsSelected->setData([
                "user_id" => $userId,
                "fields" => $fields,
                "action_type" => $actionType,
                "entity_type" => $entityType
            ]);
        }

        $this->userFieldsSelectedRepository->save($userFieldsSelected);
    }

    /**
     * Get current user
     *
     * @return \Magento\User\Model\User
     */
    private function getUser()
    {
        return $this->apiSession->getCurrentUser();
    }

    /**
     * Delete all settings of current user
     *
     * @return void
     */
    public function deleteAll()
    {
        $userId = $this->getUser()->getId();

        $userFieldsSelected = $this->userFieldsSelectedFactory->create();

        $userFieldsSelected->getCollection()
            ->addFieldToFilter('user_id', $userId)
            ->walk('delete');
    }

    /**
     * Delete all settings of current user per type
     *
     * @param string $type
     * @return void
     */
    public function deleteAllPerType($type = 'product')
    {
        $sharedTypes = [
            self::ACTION_TYPE_TONES,
            self::ACTION_TYPE_STYLE,
            self::ACTION_TYPE_CUSTOM_TONE,
            self::ACTION_TYPE_CUSTOM_STYLE,
            self::ACTION_TYPE_AUDIENCES
        ];
        $userId = $this->getUser()->getId();

        $userFieldsSelected = $this->userFieldsSelectedFactory->create();

        $userFieldsSelected->getCollection()
            ->addFieldToFilter('user_id', $userId)
            ->addFieldToFilter(['action_type', 'entity_type'], [
                ['in' => $sharedTypes],
                ['eq' => $type]
            ])
            ->walk('delete');
    }
}
