<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

namespace WriteTextAI\WriteTextAI\ViewModel;

use WriteTextAI\WriteTextAI\Model\SettingManager;
use WriteTextAI\WriteTextAI\Helper\Data;
use WriteTextAI\WriteTextAI\ViewModel\Settings;

class TooltipCategory implements \Magento\Framework\View\Element\Block\ArgumentInterface
{
    public const FAQ_URL = 'https://writetext.ai/frequently-asked-questions#credit-cost';

    /**
     * @var SettingManager
     */
    protected $settingManager;

    /**
     * @var Data
     */
    protected $helper;

    /**
     * @var Settings
     */
    protected $settings;

    /**
     * Constructor
     *
     * @param SettingManager $settingManager
     * @param Data $helper
     * @param Settings $settings
     */
    public function __construct(
        SettingManager $settingManager,
        Data $helper,
        Settings $settings
    ) {
        $this->settingManager = $settingManager;
        $this->helper = $helper;
        $this->settings = $settings;
    }

    /**
     * Get tooltip
     *
     * @param string $block
     * @return string
     */
    public function getTooltipCategory($block)
    {
        $tooltip = '';
        switch ($block) {
            case 'wtai.reviewed.category':
                $tooltip = $this->getReviewedTooltipCategory();
                break;
            case 'wtai.keywords.category':
                $tooltip = $this->getKeywordsTooltipCategory();
                break;
            case 'wtai.my.ranking.category':
                $tooltip = $this->getMyRankingTooltipCategory();
                break;
            case 'wtai.generate.rewrite.category':
                $tooltip = $this->getGenerateRewriteTooltipCategory();
                break;
            case 'wtai.audience.category':
                $tooltip = $this->getAudienceTooltipCategory();
                break;
            case 'wtai.category.description':
                $tooltip = $this->getCategoryDescriptionTooltip();
                break;
            case 'wtai.category.image':
                $tooltip = $this->getCategoryImageTooltip();
                break;
            case 'wtai.page.title.category':
                $tooltip = $this->getPageTitleTooltipCategory();
                break;
            case 'wtai.page.description.category':
                $tooltip = $this->getPageDescriptionTooltipCategory();
                break;
            case 'wtai.transfer.category':
                $tooltip = $this->getTransferTooltipCategory();
                break;
            case 'wtai.category.competitors.ranking':
                $tooltip = $this->getCompetitorsRankingTooltipCategory();
                break;
            case 'wtai.category.selected.keywords':
                $tooltip = $this->getSelectedKeywordsTooltipCategory();
                break;
        }

        return $tooltip;
    }

    /**
     * Get competitors ranking tooltip
     *
     * @return string
     */
    public function getCompetitorsRankingTooltipCategory()
    {
        return sprintf(
            '<p>%s</p>',
            __('Competitors are pages that are ranking for either the same'
                . ' keywords you’re currently ranking for or the keywords you selected'
                . ' to be included in your text. You can check which competitors are ranking'
                . ' for which keyword by clicking on the keyword. The keywords shown in this'
                . ' section are ones that your competing pages are ranking for but this category page is not.'
                . ' Data on search volume, intent, and difficulty are also displayed here.')
        ) . $this->getCategoryIntentTooltip();
    }

    /**
     * Get representative products tooltip
     *
     * @return string
     */
    public function getRepresentativesTooltip()
    {
        $rules = $this->getRules();
        $maxRepresentativeProducts = $rules['maxRepresentativeProducts'] ?? '';

        return sprintf(
            __('Select a maximum of %s products that best represent this category.'
                . ' Depending on the data available, the product name, product description, and'
                . ' product images will be used as reference for generating text for this category.'),
            $maxRepresentativeProducts
        );
    }

    /**
     * Get selected keywords tooltip
     *
     * @return string
     */
    public function getSelectedKeywordsTooltipCategory()
    {
        return sprintf(
            '<p>%s</p>',
            __('If you have selected any, WriteText.ai will retrieve the SERP data for each keyword'
                . ' (which you can preview upon clicking the keyword)'
                . ' along with intent, search volume, and difficulty.')
        ) . $this->getCategoryIntentTooltip();
    }

    /**
     * Get my ranking tooltip for category
     *
     * @return string
     */
    public function getMyRankingTooltipCategory()
    {
        return sprintf(
            '<p>%s</p>',
            __('If it’s the first time you’re doing an AI-powered keyword analysis on the site,'
                . ' WriteText.ai will retrieve ranking data for the whole domain'
                . ' (i.e., this page and your other category pages as well as product pages).'
                . ' If your page is ranking for any keyword/s, they will show here along with data on intent,'
                . ' search volume, and difficulty.')
        ) . $this->getCategoryIntentTooltip();
    }

    /**
     * Get meta title tooltip for category
     *
     * @return string
     */
    public function getPageTitleTooltipCategory()
    {
        $pageTitleLength = $this->helper->getMaxLengthSettings('page_title');

        return sprintf(__('<p>WriteText.ai aims to generate a title with around'
            . ' %s characters, based on current SEO best practices. The current'
            . ' meta title saved for the category is displayed in the box on'
            . ' the right for your reference only.</p>'), $pageTitleLength);
    }

    /**
     * Get meta description tooltip for category
     *
     * @return string
     */
    public function getPageDescriptionTooltipCategory()
    {
        $maxPageDescriptionLength = $this->helper->getMaxLengthSettings('page_description');

        return sprintf(__('<p>WriteText.ai aims to generate a description with'
            . ' around %s characters, based on current SEO best practices. The current'
            . ' meta description saved for the category is displayed in the box on'
            . ' the right for your reference only.</p>'), $maxPageDescriptionLength);
    }

    /**
     * Get generate rewrite tooltip category
     *
     * @return string
     */
    public function getGenerateRewriteTooltipCategory()
    {
        return sprintf('<p class=\"heading\">%s</p>', __('Generate new text'))
            . sprintf('<p>%s</p>', __('Generate text for your selected text types based on the settings on this page.'))
            . sprintf('<p class=\"heading\">%s</p>', __('Rewrite existing'))
            . sprintf('<p>%s</p>', __('If any of your settings'
                . ' (e.g., representative products, keywords, tone, style, and audiences)'
                . ' have changed but you don’t necessarily want to generate completely new text,'
                . ' you can use the Rewrite feature to keep the existing structure of the text but'
                . ' rewrite it to reflect any changes.'))
            . sprintf('<p>%s</p>', __('Note: Any media or directive you have inserted into the existing text'
                . ' will not be included in the rewritten version. The language of the text you'
                . ' are rewriting will be the same language used in the output.'))
            . sprintf(__('The credit cost for rewriting text will depend'
                . ' on the length of the original text and any formatting'
                . ' (e.g., bold, italics, numbered or bulleted lists) applied to it.'
                . ' For a more detailed explanation, you may read our FAQ section on <a href="%s" target="_blank">'
                . 'how your credit cost is calculated</a>.'), self::FAQ_URL);
    }

    /**
     * Get transfer tooltip for Category
     *
     * @return string
     */
    public function getTransferTooltipCategory()
    {
        return sprintf("<p>%s</p>", __("When you're done generating and/or editing text,"
                . " you have the option to save the draft inside WriteText.ai or transfer"
                . " the text to Magento. Transferring your text to Magento will publish it"
                . " on the website."))
            . sprintf("<p>%s</p>", __("Note: Any media or directive you have inserted in"
                . " your current Magento text will be overwritten when you transfer from WriteText.ai."));
    }

    /**
     * Get reviewed tooltip for category
     *
     * @return string
     */
    public function getReviewedTooltipCategory()
    {
        return sprintf('<p>%s</p>', __('Check this box to keep track of the categories'
                . ' where you have reviewed the text. This is especially helpful if you'
                . ' have an internal workflow where text needs to go through a review'
                . ' process first before being published on the website. This checkbox'
                . ' does not affect the live content, it is only a classification.'))
            . sprintf('<p>%s</p>', __('You can filter reviewed categories by'
                . ' selecting “Reviewed” under the “Filter by WriteText.ai status”'
                . ' dropdown in the category list.'));
    }

    /**
     * Get audience tooltip for category
     *
     * @return string
     */
    public function getAudienceTooltipCategory()
    {
        return __('<p>Select the applicable audience for your category so WriteText.ai'
            . ' can generate text that will appeal to them.  If you don’t select an audience,'
            . ' the generated text will default to a “neutral” audience. WriteText.ai also'
            . ' suggests more specific target markets based on your keywords. Click on the'
            . ' audience to select it or type in your own custom audience in the box.</p>');
    }

        /**
         * Get keywords tooltip for category
         *
         * @return string
         */
    public function getKeywordsTooltipCategory()
    {
        $rules = $this->getRules();
        $maxKeywords = $rules['maxKeywords'] ?? '';

        return sprintf('<p class=\"heading\">%s</p>', __('Target keywords'))
            . sprintf('<p>%s</p>', sprintf(__('Click the "Do keyword analysis" button'
                . ' to add up to %s target keywords for your category page. This will also'
                . ' open the dashboard where you can get keyword ideas as well as data on'
                . ' search volume and difficulty from Google and other search sources.'
                . ' If there are no target keywords added, WriteText.ai will generate text'
                . ' based on the category name and if selected, category image and'
                . ' representative products.'), $maxKeywords))
            . sprintf('<p class=\"heading\">%s</p>', __('Semantic keywords'))
            . sprintf('<p>%s</p>', __('Semantic keywords are words or phrases that are'
                . ' closely related to a primary keyword or topic, which helps search engines'
                . ' and users understand the context and meaning of a piece of content.'
                . ' Using semantic keywords in your content can improve its relevance and'
                . ' quality. WriteText.ai automatically suggests semantic keywords related'
                . ' to your category name and target keywords. Click on the semantic keyword'
                . ' if you want it to be considered in generating your category texts.'))
            . sprintf('<p class=\"heading\">%s</p>', __('Keyword density'))
            . sprintf('<p>%s</p>', __('Keyword density refers to the percentage of times'
                . ' your target keyword or semantic keyword appears throughout the page,'
                . ' in comparison to the total number of words in it. In WriteText.ai,'
                . ' we only take into account the text visible on your page — which is'
                . ' the category description. This means that the meta title and meta description'
                . ' are not counted.  We advise you to leave the field empty for the text type'
                . ' you are not using so that you can get an accurate keyword density.'
                . ' You can see the keyword density percentage displayed next to every keyword in this section.'));
    }

    /**
     * Get category description tooltip
     *
     * @return string
     */
    public function getCategoryDescriptionTooltip()
    {
        return __('<p>The current category description saved for the category is displayed'
                . ' in the box on the right for your reference only.')
            . __('<p>Target length<br/>')
            . __('<p>Indicate your target length by setting a minimum and maximum word count.'
                . ' WriteText.ai will aim to generate text within the number you set,'
                . ' but it may give you more or less words than expected from time to time.'
                . ' Note that your credit cost will depend on the target length you'
                . ' set so make sure to set a reasonable target range.</p>');
    }

    /**
     * Get category image tooltip
     *
     * @return string
     */
    public function getCategoryImageTooltip()
    {
        return __('<p>If available, you can let AI analyze the category image in order'
                .' to generate more accurate and relevant text. Please check that the image'
                .' accurately represents the category (i.e., it is not some kind of'
                .' placeholder or a generic image for your shop).</p>');
    }

    /**
     * Get intent tooltip
     *
     * @return string
     */
    public function getCategoryIntentTooltip()
    {
        return sprintf('<strong>%s</strong>', __('Intent guide:'))
            . sprintf(
                '<p><em>%s</em> - %s</p>',
                __('Navigational'),
                __('Users have a specific website or web page in mind and are'
                    . ' using search engines to navigate there directly.')
            )
            . sprintf(
                '<p><em>%s</em> - %s</p>',
                __('Informational'),
                __('Users seek information on a specific topic, intending to learn something.')
            )
            . sprintf(
                '<p><em>%s</em> - %s</p>',
                __('Transactional'),
                __('Users aim to complete a transaction, such as making a purchase or signing up for a service.')
            )
            . sprintf(
                '<p><em>%s</em> - %s</p>',
                __('Commercial'),
                __('Users are researching products or services with the intention of making a purchase decision soon')
            );
    }
}
