/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

define([
    'jquery',
    'ko',
    'uiComponent',
    'WriteTextAI_WriteTextAI/js/categories/model/edit/textfields',
    'WriteTextAI_WriteTextAI/js/model/edit/product',
    'WriteTextAI_WriteTextAI/js/categories/model/signalr',
    'WriteTextAI_WriteTextAI/js/model/edit',
    'WriteTextAI_WriteTextAI/js/model/edit/premium',
    'WriteTextAI_WriteTextAI/js/model/grid/prompt',
    'WriteTextAI_WriteTextAI/js/model/total-credits',
    'WriteTextAI_WriteTextAI/js/model/edit/keywords/progress',
    'WriteTextAI_WriteTextAI/js/utils/helpers',
    'wtaiSignalR',
    'mage/translate'
], function (
    $,
    ko,
    Component,
    textfields,
    product,
    signalRModel,
    editData,
    premium,
    prompt,
    totalCredits,
    keywordProgress,
    helperUtils,
    signalR,
    $t
) {
    'use strict';

    /**
     * @var {Component} self
     */
    var self;

    return Component.extend({
        defaults: {
            productId: '',
            storeId: '',
            connection: null,
            retryReconnectAttempt: 0,
            enableMaxRetryAttempt: false,
            maxRetryAttempt: 10,
            accessToken: '',
            signalRUrl: '',
            singleGenerate: 0,
            fieldStatus: 2,
            keywords: 3,
            message: $t('Text generation completed'),
            showMessage: false,
            messageSuccess: false,
            successMessage: $t('Text generation completed'),
            pageTitle: '',
            pageDescription: '',
            categoryDescription: '',
            errorMessage: '',
            statusFailed: 'Failed',
            statusCompleted: 'Completed'
        },

        isGettingNewToken: false,

        /** @inheritdoc */
        initialize: function () {
            this._super();

            self = this;

            this.connectTostream();

            product.productId.subscribe(function (productId) {
                self.productId(productId);
            });

            product.storeId.subscribe(function (storeId) {
                self.storeId(storeId);
            });

            signalRModel.showMessage.subscribe(function (show) {
                self.showMessage(show);
            });

            signalRModel.message.subscribe(function (message) {
                self.message(message);
            });
        },

        /** @inheritdoc */
        initObservable: function () {
            this._super().observe([
                'productId',
                'storeId',
                'accessToken',
                'signalRUrl',
                'message',
                'showMessage',
                'messageSuccess',
                'errorMessage'
            ]);

            this.generateComplete = ko.computed(function () {
                var fieldsGenerated = signalRModel.fieldsGenerated(),
                    fieldsValues = Object.values(fieldsGenerated),
                    hasTrue = fieldsValues.includes(true),
                    hasFalse = fieldsValues.includes(false);

                var isGenerating = signalRModel.generating(),
                    isTextsGenerating = hasTrue || hasFalse,
                    isTextsGenerated = hasTrue && !hasFalse;

                if (isGenerating && isTextsGenerating && isTextsGenerated) {
                    return true;
                }

                return false;
            });

            this.generateComplete.subscribe(function (generateComplete) {
                if (generateComplete) {
                    self.catchFieldsError();
                    self.getGenerated();
                }
            });

            this.keywordsOngoing = ko.computed(function () {
                return Object.keys(keywordProgress.status()).length > 0 &&
                    keywordProgress.status().status !== 'Completed';
            }, this);

            return this;
        },

        /**
         * Catch fields error.
         */
        catchFieldsError: function () {
            self.errorMessage('');
            self.messageSuccess(true);
            self.getNormalTextsError();
        },

        /**
         * Get normal texts error.
         */
        getNormalTextsError: function () {
            var errorFields = signalRModel.errorFields(),
                selectedFields = signalRModel.selectedFields(),
                htmlMessage = helperUtils.sprintf(
                    $t('Error encountered while generating the following text: %s'),
                    '<ul>' + errorFields.map(field => `<li>${field}</li>`).join('') + '</ul>'
                );

            if (selectedFields.length > 0 && selectedFields.length === errorFields.length) {
                self.messageSuccess(false);
                signalRModel.message(signalRModel.errorMessage());
            } else if (errorFields.length > 0) {
                signalRModel.message(self.successMessage);
                self.errorMessage(htmlMessage);
            }
        },

        /**
         * Hide text generation success message.
         */
        hideMessage: function () {
            signalRModel.showMessage(false);
        },

        /**
         * Connect to signalR stream.
         */
        connectTostream: function () {
            if (self.accessToken()) {
                self.connection = new signalR.HubConnectionBuilder()
                    .withUrl(self.signalRUrl(), {
                        accessTokenFactory: () => self.accessToken()
                    })
                    .build();
                self.connection
                    .start()
                    .then(() => self.onConnected(self.connection))
                    .catch((error) => self.onConnectionError(error));

                self.bindConnectionMessageGenerate(self.connection);

                $(window).on('focus', function () {
                    if (
                        self.connection != null &&
                        self.connection.connectionState != "Connected" &&
                        self.connection.connectionState != "Connecting"
                    ) {
                        self.connection.start();
                    }
                });
            }
        },

        /**
         * Connection is established actions.
         *
         * @param {object} connection
         * @returns {void}
         */
        onConnected: function (connection) {
            connection.send('broadcastMessage', '_SYSTEM_', 'Connected');
        },

        /**
         * Connection error actions.
         *
         * @param {object} error
         * @returns {void}
         */
        onConnectionError: function (error) {
            var doConnect = true;

            if (error.statusCode === 401 && !self.isGettingNewToken) {
                self.getNewAccessToken();
            }

            if (self.enableMaxRetryAttempt && self.retryReconnectAttempt >= self.maxRetryAttempt) {
                doConnect = false;
            }

            if (doConnect) {
                if (self.retryReconnectAttempt === 0) {
                    self.connectTostream();
                } else {
                    setTimeout(function () {
                        self.connectTostream(); /* attempt to connect every 5 seconds */
                    }, 5000);
                }

                self.retryReconnectAttempt++;
            }
        },

        /**
         * Get new access token.
         */
        getNewAccessToken: function () {
            var url = self.getNewAccessTokenUrl;
        
            self.isGettingNewToken = true;
            $.ajax({
                url: url,
                type: 'GET',
                dataType: 'json',
                showLoader: false,
                success: function (response) {
                    if (response.success) {
                        self.accessToken(response.access_token);
                    }
                }
            }).always(function () {
                self.isGettingNewToken = false;
            });
        },

        /**
         * Bind connection messages.
         *
         * @param {Object} connection
         * @returns {void}
         */
        bindConnectionMessageGenerate: function (connection) {
            if (connection === null) {
                /* bypass */
            }

            connection.on('broadcastMessage', self.messageCallback);
            connection.onclose(self.onConnectionError);
        },

        /**
         * Parse message.
         *
         * @param {String} encodedName
         * @param {String} encodedMsg
         *
         * @returns {Object}
         */
        createMessageEntry: function (encodedName, encodedMsg) {
            return {
                encodedName: encodedName,
                encodedMsg: JSON.parse(encodedMsg)
            };
        },

        /**
         * Update text fields.
         *
         * @param {String} field
         * @param {String} value
         *
         * @returns {void}
         */
        updateField: function (field, value) {
            switch (field) {
                case self.pageTitle:
                    textfields.pageTitle(value);
                    break;
                case self.pageDescription:
                    textfields.pageDescription(value);
                    break;
                case self.categoryDescription:
                    /** textfields.categoryDescription(value); */
                    break;
            }
        },

        /**
         * Get text field value.
         *
         * @param {String} field
         * @returns {String}
         */
        getField: function (field) {
            var value = '';

            switch (field) {
                case self.pageTitle:
                    value = textfields.pageTitle();
                    break;
                case self.pageDescription:
                    value = textfields.pageDescription();
                    break;
                case self.categoryDescription:
                    value = textfields.categoryDescription();
                    break;
            }

            return value;
        },

        /**
         * Get index field.
         *
         * @param {String} field
         * @returns {String}
         */
        getIndexField: function (field) {
            var value = '';

            switch (field) {
                case self.pageTitle:
                    value = 'page_title';
                    break;
                case self.pageDescription:
                    value = 'page_description';
                    break;
                case self.categoryDescription:
                    value = 'category_description';
                    break;
            }

            return value;
        },

        /**
         * Process stream message.
         *
         * @param {String} name
         * @param {String} message
         *
         * @returns {void}
         */
        messageCallback: function (name, message) {
            var encodedName = name,
                encodedMsg = message
                    .replace(/&/g, '&amp;')
                    .replace(/</g, '&lt;')
                    .replace(/>/g, '&gt;'),
                messageEntry = self.createMessageEntry(encodedName, encodedMsg),
                streamType = messageEntry.encodedMsg.type,
                streamFieldType = messageEntry.encodedMsg.field,
                streamStop = messageEntry.encodedMsg.stop,
                streamPartialText = messageEntry.encodedMsg.partialText,
                streamRecordId = messageEntry.encodedMsg.recordId,
                streamStoreId = messageEntry.encodedMsg.storeId,
                streamIndex = messageEntry.encodedMsg.index,
                generationType = messageEntry.encodedMsg.generationType,
                indexField = '',
                streamStatus = '',
                fieldsGenerated = {},
                productId = self.productId(),
                storeId = self.storeId();

            console.log(messageEntry.encodedMsg);
            
            switch (streamType) {
                case self.fieldStatus:
                    if (signalRModel.generating() === true) {
                        indexField = self.getIndexField(streamFieldType);
                        streamStatus = messageEntry.encodedMsg.status;
        
                        if (streamStatus === self.statusFailed) {
                            signalRModel.errorFields.push(streamFieldType);
                            signalRModel.errorMessage(messageEntry.encodedMsg.error);
                        }
        
                        fieldsGenerated = signalRModel.fieldsGenerated();
                        fieldsGenerated[indexField] = true;
                        signalRModel.fieldsGenerated(fieldsGenerated);
                        if (productId === streamRecordId && storeId === streamStoreId) {
                            var statuses = textfields.statuses();
                            switch (indexField) {
                                case 'page_title':
                                    $('.wtai-page-title-loading-mask').removeClass('wtai-show');
                                    statuses['pageTitleGenerateStatus'] = true;
                                    break;
                                case 'page_description':
                                    $('.wtai-page-description-loading-mask').removeClass('wtai-show');
                                    statuses['pageDescriptionGenerateStatus'] = true;
                                    break;
                                case 'category_description':
                                    $('.wtai-category-description-loading-mask').removeClass('wtai-show');
                                    statuses['categoryDescriptionGenerateStatus'] = true;
                                    break;
                            }
                            
                            textfields.statuses(statuses);
                        }
                    }
                    if (this.keywordsOngoing() && streamStatus === self.statusFailed) {
                        keywordProgress.status({
                            status: self.statusFailed,
                            status_display: ''
                        });
                    }
                    break;
                case self.singleGenerate:
                    if (signalRModel.generating() === true) {
                        indexField = self.getIndexField(streamFieldType);
                        if (productId === streamRecordId && storeId === streamStoreId) {
                            /* eslint-disable max-depth */
        
                            if (streamIndex === signalRModel.index()[indexField]) {
                                self.processSingleGenerate(
                                    streamFieldType,
                                    streamStop,
                                    streamPartialText,
                                    streamIndex
                                );
                                streamIndex += 1;
                                signalRModel.index()[indexField] = streamIndex;
        
                                while (
                                    typeof signalRModel.messages()[indexField][
                                        streamIndex
                                    ] !== 'undefined'
                                ) {
                                    self.processSingleGenerate(
                                        signalRModel.messages()[indexField][streamIndex]
                                            .encodedMsg.field,
                                        signalRModel.messages()[indexField][streamIndex]
                                            .encodedMsg.stop,
                                        signalRModel.messages()[indexField][streamIndex]
                                            .encodedMsg.partialText,
                                        signalRModel.messages()[indexField][streamIndex]
                                            .encodedMsg.index
                                    );
                                    streamIndex += 1;
                                    signalRModel.index()[indexField] = streamIndex;
                                }
                            } else {
                                signalRModel.messages()[indexField][streamIndex] =
                                    messageEntry;
                            }
        
                            if (streamStop === true) {
                                /* generate end */
                                signalRModel.index()[indexField] = 0;
                                signalRModel.messages()[indexField] = {};
                            }
                            /* eslint-enable max-depth */
                        }
                    }
                    break;
                case self.keywords:
                    keywordProgress.status(messageEntry.encodedMsg);
                    break;
            }
        },

        /**
         * Process single generate.
         *
         * @param {String} streamFieldType
         * @param {Boolean} streamStop
         * @param {String} streamPartialText
         * @param {Number} streamIndex
         *
         * @returns {void}
         */
        processSingleGenerate: function (
            streamFieldType,
            streamStop,
            streamPartialText,
            streamIndex
        ) {
            var currentValue = self.getField(streamFieldType),
                newValue = currentValue + streamPartialText;

            /* generate starts */
            if (streamIndex === 0) {
                self.updateField(streamFieldType, streamPartialText);
            }

            /* generate partial */
            if (
                streamPartialText !== null &&
                streamIndex !== null &&
                streamIndex !== 0 &&
                streamStop === false
            ) {
                self.updateField(streamFieldType, newValue);
            }
        },

        /**
         * Get generated text fields.
         */
        getGenerated: function () {
            var url = self.getGeneratedUrl,
                statuses = textfields.statuses(),
                productId = self.productId(),
                storeId = self.storeId(),
                selectedFields = signalRModel.selectedFields() || [];

            $.ajax({
                url: url,
                type: 'POST',
                data: {
                    category_id: productId,
                    store_id: storeId
                },
                dataType: 'json',
                showWriteTextAILoader: true,
                showPageTitleLoader: selectedFields.includes('page_title'),
                showPageDescriptionLoader: selectedFields.includes('page_description'),
                showCategoryDescriptionLoader: selectedFields.includes('category_description'),
                success: function (response) {
                    if (typeof response.is_premium !== 'undefined') {
                        premium.isPremium(response.is_premium);
                        prompt.isPremium(response.is_premium);
                        totalCredits.totalCredits(response.total_credits);
                    }
                    
                    if (response.success) {
                        selectedFields.forEach((field) => {
                            switch (field) {
                                case 'page_title':
                                    textfields.pageTitle(response.generated_texts.page_title);
                                    textfields.originalPageTitle(response.generated_texts.page_title);
                                    statuses['pageTitleGenerateStatus'] = true;
                                    statuses['pageTitleTransferStatus'] = false;
                                    $('#wtaiPageTitleMask').addClass('wtai-generated');
                                    break;
                                case 'page_description':
                                    textfields.pageDescription(response.generated_texts.page_description);
                                    textfields.originalPageDescription(response.generated_texts.page_description);
                                    statuses['pageDescriptionGenerateStatus'] = true;
                                    statuses['pageDescriptionTransferStatus'] = false;
                                    $('#wtaiPageDescMask').addClass('wtai-generated');
                                    break;
                                case 'category_description':
                                    textfields.categoryDescription(response.generated_texts.category_description);
                                    textfields.originalCategoryDescription(response.generated_texts.category_description);
                                    statuses['categoryDescriptionGenerateStatus'] = true;
                                    statuses['categoryDescriptionTransferStatus'] = false;
                                    $('#wtaiCategoryDescMask + .mce-tinymce').addClass('wtai-generated');
                                    $('#wtaiCategoryDescMask + .tox-tinymce').addClass('wtai-generated');
                                    break;
                            }
                            textfields.statuses(statuses);
                        });

                        if (editData.opened()) {
                            signalRModel.showMessage(true);
                            self.showMessage(true);
                        }
                        signalRModel.generating(false);
                    } else {
                        console.log(response.message);
                    }
                }
            });
        }
    });
});
