/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

define([
    'jquery',
    'ko',
    'uiComponent',
    'underscore',
    'WriteTextAI_WriteTextAI/js/model/edit/keywords',
    'WriteTextAI_WriteTextAI/js/model/edit/product',
    'WriteTextAI_WriteTextAI/js/model/edit/premium',
    'WriteTextAI_WriteTextAI/js/model/grid/prompt',
    'WriteTextAI_WriteTextAI/js/model/total-credits',
    'WriteTextAI_WriteTextAI/js/model/edit/audience',
    'WriteTextAI_WriteTextAI/js/model/edit/keywords/progress',
    'WriteTextAI_WriteTextAI/js/utils/edit/keywords',
    'mage/translate'
], function (
    $,
    ko,
    Component,
    _,
    keywords,
    product,
    premium,
    prompt,
    totalCredits,
    audience,
    keywordProgress,
    keywordUtils,
    $t
) {
    'use strict';

    /**
     * @var {Component} self
     */
    var self;

    return Component.extend({
        defaults: {
            generalMessage: $t('A system error has occurred. Please click the "Start AI-powered analysis" button again. If the issue persists, please contact our support team at support@writetext.ai.'),
            removeKeywordOngoing: false,
        },

        /** @inheritdoc */
        initialize: function () {
            this._super();

            self = this;

            $('.wtai-keyword-analysis-selected > *').applyBindings(this);

            keywordUtils.initToggleSerpPopup('.wtai-keyword-analysis-selected', '.wtai-serp-popup');
        },

        /** @inheritdoc */
        initObservable: function () {
            this._super()
                .observe([
                    'removeKeywordOngoing'
                ]);

            this.serpDate = ko.computed(function () {
                return keywords.serpDate();
            });

            this.targetKeywords = ko.computed(function () {
                return keywords.targetKeywords();
            });

            this.getFilteredKeywords = ko.computed(function () {
                return this.targetKeywords().filter(function (keyword) {
                    return self.keywordRemovable(keyword.text);
                });
            }, this);

            this.keywordsCount = ko.computed(function () {
                var keywords = this.getFilteredKeywords(),
                    count = keywords ? keywords.length : 0;
                return count + '/' + this.rules.maxKeywords;
            }, this);

            this.showTable = ko.computed(function () {
                var keywords = this.getFilteredKeywords();
                return keywords.length >= 1;
            }, this);

            this.showEmpty = ko.computed(function () {
                var keywords = this.getFilteredKeywords();
                return keywords.length < 1;
            }, this);
            
            this.status = ko.computed(function () {
                return keywordProgress.status();
            });

            this.showProgress = ko.computed(function () {
                var keywords = this.getFilteredKeywords();

                return Object.keys(this.status()).length > 0 &&
                    keywordProgress.selectedAndCompetitorProgressVisible() &&
                    this.status().status !== 'Completed' &&
                    keywords.length >= 1;
            }, this);

            this.getMessage = keywordUtils.getMessage(this);

            this.getCount = keywordUtils.getCount(this);

            this.getPercent = keywordUtils.getPercent(this);

            this.isLoading = ko.computed(function () {
                var keywords = this.getFilteredKeywords();
                
                return keywordProgress.selectedAndCompetitorLoading() &&
                keywords.length >= 1;
            }, this);

            this.analysisInProgress = keywordUtils.analysisInProgress();

            this.keywordsChangeOngoing = ko.computed(function () {
                return keywords.keywordsChangeOngoing();
            });

            this.isSectionDisabled = ko.computed(function () {
                return keywords.ideasSortFilterOngoing() ||
                    keywords.ideasShowMoreOngoing() ||
                    keywords.keywordsChangeOngoing() ||
                    keywords.manualKeywordsChangeOngoing() ||
                    keywordProgress.ideasLoading();
            });

            this.showRefresh = ko.computed(function () {
                return keywords.showSelectedRefresh();
            }, this);

            return this;
        },

        /**
         * Select spell
         *
         * @param {Object} data
         * @param {Event} event
         *
         * @returns {void}
         */
        selectSpell: function (data, event) {
            var element = event.currentTarget,
                parentElement = $(element).closest('td.wtai-keyword'),
                newKeyword = data.spell,
                oldKeyword = data.text;

            event.stopPropagation();

            keywords.keywordsChangeOngoing(true);

            $(parentElement).addClass('wtai-loading');

            keywords.errorMessage('');

            $.ajax({
                url: self.spellCheckUrl,
                type: 'POST',
                data: {
                    entity_type: self.entityType,
                    product_id: product.productId(),
                    store_id: product.storeId(),
                    new_keyword: newKeyword,
                    old_keyword: oldKeyword,
                    location: keywords.location(),
                    keywords: keywords.targetKeywords()
                        .map(function (keyword) {
                            return keyword.text;
                        })
                },
                dataType: 'json',
                showWriteTextAILoader: false,
                success: function (response) {
                    var target = {},
                        newTarget = {},
                        manual = {},
                        newManual = {};

                    $(parentElement).removeClass('wtai-loading');

                    if (typeof response.is_premium !== 'undefined') {
                        premium.isPremium(response.is_premium);
                        prompt.isPremium(response.is_premium);
                        totalCredits.totalCredits(response.total_credits);
                    }
                    
                    if (response.success) {
                        /** Update selected keyword */
                        target = ko.utils.arrayFirst(
                            keywords.targetKeywords(),
                            function (targetData) {
                                return targetData.text === oldKeyword;
                            }
                        );

                        if (target) {
                            newTarget = {...target};
                            newTarget.values = response.data.values;
                            newTarget.text = newKeyword;
                            keywords.targetKeywords.replace(target, newTarget);
                            keywords.targetKeywords.valueHasMutated();
                        }
                        /** End Update selected keyword */

                        /** Update manual keyword */
                        manual = ko.utils.arrayFirst(
                            keywords.manualInput(),
                            function (manualData) {
                                return manualData.keyword === oldKeyword;
                            }
                        );

                        if (manual) {
                            newManual = {...manual};
                            newManual.keyword = newKeyword;
                            keywords.manualInput.replace(manual, newManual);
                            keywords.manualInput.valueHasMutated();
                        }
                        /** End Update manual keyword */

                        audience.selectedAudience(response.selected_audience.shift());
                        audience.suggestedAudience(response.suggested_audience);
                        keywords.keywordsChangeOngoing(false);
                    } else {
                        keywords.errorMessage(response.message);
                    }
                }.bind(this)
            });
        },

        /**
         * Get domain
         *
         * @param {String} domain
         * @returns {String}
         */
        getDomain: function (domain) {
            if (!domain.startsWith('http://') && !domain.startsWith('https://')) {
                domain = 'https://' + domain;
            }
            return domain;
        },

        /**
         * Dismiss
         */
        dismiss: function () {
            keywordProgress.status({});
            keywordProgress.selectedAndCompetitorLoading(false);
            keywordProgress.selectedAndCompetitorProgressVisible(false);
        },
    
        /**
         * Refresh data.
         */
        refresh: function () {
            keywords.successMessage('');
            keywordProgress.selectedAndCompetitorLoading(true);
            keywordProgress.selectedAndCompetitorProgressVisible(true);
            keywords.errorMessage('');
            $.ajax({
                url: self.refreshUrl,
                type: 'POST',
                data: {
                    entity_type: self.entityType,
                    product_id: product.productId(),
                    product_name: product.productName(),
                    store_id: product.storeId(),
                    product_url: product.productUrl(),
                    keywords: keywords.manualInput().map(input => input.keyword),
                    target_keywords: keywords.targetKeywords()
                        .map(input => input.text)
                        .filter(keyword => self.keywordRemovable(keyword)),
                    location: keywords.location()
                },
                dataType: 'json',
                showWriteTextAILoader: false,
                showAnalysisLoader: false,
                success: function (response) {
                    if (typeof response.is_premium !== 'undefined') {
                        premium.isPremium(response.is_premium);
                        prompt.isPremium(response.is_premium);
                        totalCredits.totalCredits(response.total_credits);
                    }
                    if (!response.success) {
                        keywords.errorMessage(response.message);
                        keywordProgress.selectedAndCompetitorLoading(false);
                        keywordProgress.allSectionsLoading(false);
                        keywordProgress.ideasLoading(false);
                        $('.wtai-analysis-loading-mask').removeClass('wtai-show');
                    }
                }
            });
        },
        
        /**
         * Check if domain is current site.
         *
         * @param {string} domain
         * @param {string} type
         *
         * @returns {boolean}
         */
        isMySite: function (domain, type) {
            return domain === window.location.hostname && type !== 'featured_snippet';
        },

        /**
         * Check if keyword is removable.
         *
         * @param {string} keyword
         * @returns {boolean}
         */
        keywordRemovable: function (keyword) {
            return keywordUtils.keywordRemovable(keyword);
        },

        /**
         * Close modal.
         */
        closeModal: function () {
            $(self.modalSelector).modal('closeModal');
        },

        /**
         * Remove keyword from product.
         *
         * @param {Object} data
         * @param {Event} event
         *
         * @returns {void}
         */
        removeKeyword: function (data, event) {
            var element = event.currentTarget,
            parentElement = $(element).closest('tr');

            if (self.isSectionDisabled()) {
                return;
            }
            
            if (!premium.isPremium()) {
                self.closeModal();
                return;
            }

            $(parentElement).find('.wtai-action').addClass('wtai-loading');
            self.removeKeywordOngoing(true);

            keywords.errorMessage('');
            keywords.targetKeywords.remove(this);

            $.ajax({
                url: self.keywordDeleteUrl,
                type: 'POST',
                data: {
                    entity_type: self.entityType,
                    product_id: product.productId(),
                    store_id: product.storeId(),
                    remaining_keywords: keywords.targetKeywords()
                        .filter(function (keyword) {
                            return keyword.text !== self.text;
                        })
                        .map(function (keyword) {
                            return { text: keyword.text };
                        })
                },
                dataType: 'json',
                showWriteTextAILoader: true,
                hideWriteTextTitleLoader: true,
                showKeywordsChangeLoader: true,
                success: function (response) {
                    $(parentElement).find('.wtai-action').removeClass('wtai-loading');
                    self.removeKeywordOngoing(false);
                    if (typeof response.is_premium !== 'undefined') {
                        premium.isPremium(response.is_premium);
                        prompt.isPremium(response.is_premium);
                        totalCredits.totalCredits(response.total_credits);
                    }
                    
                    if (response.success) {
                        keywords.selectedSemanticKeywords(response.selected);
                        audience.selectedAudience(response.selected_audience.shift());
                        audience.suggestedAudience(response.suggested_audience);
                        keywords.targetRemoved(true);
                    } else {
                        keywords.errorMessage(response.message);
                        keywords.targetKeywords.push(this);
                    }
                }.bind(this)
            });
        },

    });
});
